@@ -4,8 +4,8 @@ import os
 
 SCORE_FILE=os.environ['HOME']+'/.shisensho-scores-v2'
 SCORE_LIMIT=15
-VERSION='0.7'
-VERSION_DATE='2013-04-23'
+VERSION='0.8'
+VERSION_DATE='2013-08-15'
 
 TILES_NAMES=['Classic', 'Modern']
 TILES_NAMES_SCORE=['classic', 'modern']
@@ -30,6 +30,11 @@ import operator
 import time
 import string
 import random
+'''
+ portrait = (event.width < event.height)
+self.window.connect('configure-event', reorder_main)
+
+'''
 #import code ## only for debug:  code.InteractiveConsole(locals()).interact()
 try:
     import hildon
@@ -103,6 +108,7 @@ class ShishenShoModel:
     def cell_at(self,x,y):
         return self.board[y][x]
     def can_move(self,x1,y1,x2,y2):
+        #print self.board
         if self.gamedone: return False
         if x1==x2 and y1==y2: return False
         if self.cell_at(x1,y1)==None: return False
@@ -253,13 +259,17 @@ class ShishenShoRenderer:
 
 
 
+
 class ShishenShoGame:
     def __init__(self):
         try:
             self.window = hildon.Window()
+            hildon.hildon_gtk_window_set_portrait_flags(self.window, hildon.PORTRAIT_MODE_SUPPORT)
+            self.window.connect('configure-event', self.orientation_changed)
         except NameError:
             self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
         self.window.set_title("Shisen Sho")
+
         self.window.connect("destroy", lambda w: gtk.main_quit())
         self.area = gtk.DrawingArea()
         self.area.set_size_request(670, 480)
@@ -296,12 +306,12 @@ class ShishenShoGame:
 
         self.button_score = gtk.Button(label="Scores")
         self.button_about = gtk.Button(label="About")
-        #self.button_quit = gtk.Button(label="Quit")
+        self.button_quit = gtk.Button(label="Quit")
         self.button_new.connect("button_press_event", self.action_new)
         self.button_pause.connect("button_press_event", self.action_pause)
         self.button_score.connect("button_press_event", self.action_score)
         self.button_about.connect("button_press_event", self.action_about)
-        #self.button_quit.connect("button_press_event", self.action_quit)
+        self.button_quit.connect("button_press_event", self.action_quit)
         buttonbox.pack_start(self.label_info, False, False,1)
         buttonbox.pack_start(self.button_new, False, False,1)
 
@@ -329,7 +339,7 @@ class ShishenShoGame:
         buttonbox.pack_start(self.button_pause, False, False,1)
         buttonbox.pack_start(self.button_score, False, False,1)
         buttonbox.pack_start(self.button_about, False, False,1)
-        #buttonbox.pack_start(self.button_quit, False, False,5)
+        buttonbox.pack_start(self.button_quit, False, False,1)
         buttonbox.show()
         buttonbox.set_size_request(130, 480)
         hbox.pack_end(buttonbox, False, False, 5)
@@ -347,6 +357,61 @@ class ShishenShoGame:
         self.clear_path_ID=None
         self.activate_clear_path()
         gobject.timeout_add(1000, self.update_timer)
+
+    def orientation_changed(self,widget, event):
+        #print self.window.get_size()
+        width, height = self.window.get_size()
+        portrait = (width < height)
+        #print "screen portrait :",portrait
+
+        board_portrait = (self.model.width < self.model.height)
+        if board_portrait is portrait:
+           return
+
+        #print self.model.board
+
+        #print self.model.width
+        #print self.model.height
+
+
+        #backup:
+        temp_board = [None]*self.model.height
+        for i in range(len(self.model.board)):
+            temp_board[i]=[0]*self.model.width
+
+
+        for i in range(self.model.height):     # height
+            for j in range(self.model.width):  # width
+                  temp_board[i][j] =self.model.board[i][j]
+
+        #print temp_board
+
+
+ 
+        #re-arrange:
+        self.model.width,self.model.height=self.model.height,self.model.width
+        self.model.board=[None]*self.model.height
+
+        for i in range(len(self.model.board)):
+            self.model.board[i]=[0]*self.model.width
+
+        for i in range(self.model.width): #OLD HEIGHT
+            for j in range(self.model.height): #OLD WIDTH
+                  self.model.board[j][i]=temp_board[i][j]
+
+        '''
+        print "-----------"
+        print self.model.board   
+
+        print self.model.width
+        print self.model.height
+        '''
+
+        #if there are selection, swap x and y of it (then same card keep selection)
+        if self.markedcell!=None:
+           self.markedcell =(self.markedcell[1],self.markedcell[0])
+
+
     def update_timer(self):
         self.label_info.set_text("Time: %2d:%02d\nTiles left: %d"%
                                  (divmod(int(self.model.get_time()),60)+(self.model.moves_left(),)))
@@ -380,6 +445,9 @@ class ShishenShoGame:
         if rr or nr!=self.tilesetnr:
             self.tilesetnr=nr
             self.reload_renderer()
+        self.orientation_changed(widget,event)
+
+
         self.redraw()
     def action_pause(self, widget, event):
         if self.model.is_pause():
@@ -440,6 +508,10 @@ name and board type description is noted in score table.
 Game can be also stuck in state where no move is possible.
 """
         text3 = """
+Version: 0.8
+Aapo Rantalainen: *Portrait mode support (for N900/Maemo5)
+ *checking if there aren't valid pairs left
+--------------------
 Version: 0.7
 Aapo Rantalainen slightly modified game for Nokia N900/Maemo5.
 --------------------
@@ -463,6 +535,7 @@ the result.
             scroll = gtk.ScrolledWindow()
             scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
             tv = gtk.TextView()
+            tv.set_wrap_mode(gtk.WRAP_WORD)
             tv.set_editable(False)
             tb = tv.get_buffer()
             scroll.add(tv)
@@ -479,8 +552,8 @@ the result.
         dialog.vbox.pack_start(notebook, True, True, 0)
         dialog.run()
         dialog.destroy()
-    #def action_quit(self, widget, event):
-    #    gtk.main_quit()
+    def action_quit(self, widget, event):
+        gtk.main_quit()
     def decodexy(self, ax, ay):
         return self.renderer.decodexy(self.area, self.model, ax, ay)
     def add_score(self):
@@ -600,6 +673,41 @@ the result.
         dialog.vbox.pack_start(notebook, True, True, 0)
         dialog.run()
         dialog.destroy()
+
+
+    def check_if_there_are_valid_pairs_left(self):
+        '''
+         Loop over all non-empty cells and check if it is valid pair with any 
+         of the remaining cell. In worst case this is O(n^2), where n is number 
+         of cells. 
+
+         But in real:
+         *(currently) maximum count of cell is 7*14 in hard
+         *check is called after first pair is taken
+         *n=7*14-2=96
+         *first cell is checked against 95 cells
+         *second cell is checked against 94 cells
+         *...
+         *95th cell is checked agaist 1 cell
+         *96th cell is checked agaist 0 cells
+ 
+         *for i in range(95): s = s + i
+           ->4465 which is not problem at all
+
+        '''
+        for y1 in range(self.model.height):      # height
+            for x1 in range(self.model.width):    # width
+                if self.model.cell_at(x1,y1)!=None: 
+                    for y2 in range(y1,self.model.height):
+                        for x2 in range(x1,self.model.width):
+                            if self.model.can_move(x1,y1,x2,y2): #if returns anything but False
+                                #print "hint: ",x1,y1,x2,y2
+                                return True
+        
+        #no valid pairs found
+        return False
+
+
     def area_press_event(self, widget, event):
         if self.model.is_pause(): return
         pos=self.decodexy(int(event.x), int(event.y))
@@ -625,6 +733,17 @@ the result.
                                 dialog.run()
                                 dialog.destroy()
                                 self.add_score()
+
+                            else:
+                                if not self.check_if_there_are_valid_pairs_left():
+                                   dialog = gtk.MessageDialog(self.window,
+                                                           gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                                           gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
+                                                           "No pairs left, You lose")
+                                   dialog.run()
+                                   dialog.destroy()
+
+
                         else:
                             self.markedcell=pos
         self.redraw()
